﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace memcached.Tx
{
    /// <summary>
    /// We assume default(T) is not a valid value. 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal sealed class GetOp<T> : ReadOnlyOp
    {
        [System.Diagnostics.DebuggerHidden]
        public GetOp(ICache real, ICache temp, string key)
            : base(real, temp, key)
        {
        }

        public bool Success;

        public override string Name
        {
            [System.Diagnostics.DebuggerHidden]
            get { return "Get:" + Key; }
        }

        public override object Execute()
        {
            // get the current Volatile value if exists, or the Committed one
            object current = Temp.Get<T>(Key, out Success);// this will always succeed
            object nil = default(T);
            if (!Success || current == nil)
                //TODO for repeatable read, store the value in Temp?
                return Store.Get<T>(Key, out Success);
            return (T)current;
        }
    }
}